@@ -23,8 +23,9 @@ from branca.colormap import linear, ColorMap
 
 from ._version import EXTENSION_VERSION
 
+from .projections import projections
+
 def_loc = [0.0, 0.0]
-allowed_crs = ['Earth', 'EPSG3395', 'EPSG3857', 'EPSG4326', 'Base', 'Simple']
 allowed_cursor = ['alias', 'cell', 'grab', 'move', 'crosshair', 'context-menu',
                   'n-resize', 'ne-resize', 'e-resize', 'se-resize', 's-resize',
                   'sw-resize', 'w-resize', 'nw-resize', 'nesw-resize',
@@ -299,7 +300,7 @@ class WMSLayer(TileLayer):
     styles = Unicode().tag(sync=True, o=True)
     format = Unicode('image/jpeg').tag(sync=True, o=True)
     transparent = Bool(False).tag(sync=True, o=True)
-    crs = Enum(values=allowed_crs, default_value='EPSG3857').tag(sync=True)
+    crs = Dict(default_value=projections.EPSG3857).tag(sync=True)
     uppercase = Bool(False).tag(sync=True, o=True)
 
 
@@ -550,7 +551,9 @@ class GeoJSON(FeatureGroup):
 
         properties = feature['properties']
         if 'style' in properties:
-            properties['style'].update(style_callback(feature))
+            style = properties['style'].copy()
+            style.update(style_callback(feature))
+            properties['style'] = style
         else:
             properties['style'] = style_callback(feature)
 
@@ -601,11 +604,6 @@ class Choropleth(GeoJSON):
     colormap = Instance(ColorMap)
     key_on = Unicode('id')
 
-    @observe('choro_data')
-    def _update_bounds(self, change):
-        self.value_min = min(self.choro_data.items(), key=lambda x: x[1])[1]
-        self.value_max = max(self.choro_data.items(), key=lambda x: x[1])[1]
-
     @observe('style', 'style_callback', 'value_min', 'value_max', 'geo_data', 'choro_data', 'colormap')
     def _update_data(self, change):
         self.data = self._get_data()
@@ -929,6 +927,22 @@ class LegendControl(Control):
         self.send_state()
 
 
+class SearchControl(Control):
+    _view_name = Unicode('LeafletSearchControlView').tag(sync=True)
+    _model_name = Unicode('LeafletSearchControlModel').tag(sync=True)
+
+    url = Unicode().tag(sync=True, o=True)
+    zoom = Int(10).tag(sync=True, o=True)
+    property_name = Unicode('display_name').tag(sync=True, o=True)
+    property_loc = List(['lat', 'lon']).tag(sync=True, o=True)
+    jsonp_param = Unicode('json_callback').tag(sync=True, o=True)
+    auto_type = Bool(False).tag(sync=True, o=True)
+    auto_collapse = Bool(False).tag(sync=True, o=True)
+    animate_location = Bool(False).tag(sync=True, o=True)
+
+    marker = Instance(Marker).tag(sync=True, **widget_serialization)
+
+
 class MapStyle(Style, Widget):
     """ Map Style Widget """
     _model_name = Unicode('LeafletMapStyleModel').tag(sync=True)
@@ -948,6 +962,9 @@ class Map(DOMWidget, InteractMixin):
     _view_module_version = Unicode(EXTENSION_VERSION).tag(sync=True)
     _model_module_version = Unicode(EXTENSION_VERSION).tag(sync=True)
 
+    # URL of the window where the map is displayed
+    window_url = Unicode(read_only=True).tag(sync=True)
+
     # Map options
     center = List(def_loc).tag(sync=True, o=True)
     zoom_start = CFloat(12).tag(sync=True, o=True)
@@ -955,7 +972,7 @@ class Map(DOMWidget, InteractMixin):
     max_zoom = CFloat(18).tag(sync=True, o=True)
     min_zoom = CFloat(1).tag(sync=True, o=True)
     interpolation = Unicode('bilinear').tag(sync=True, o=True)
-    crs = Enum(values=allowed_crs, default_value='EPSG3857').tag(sync=True)
+    crs = Dict(default_value=projections.EPSG3857).tag(sync=True)
 
     # Specification of the basemap
     basemap = Union(
